/* 
  Magnum 
  Copyright (C) 2002-2004 Kaya Memisoglu. All Rights Reserved.

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, under the terms in LICENSE.TXT.

  Kaya Memisoglu  (k.memisoglu@dimajix.de)
*/

#include "graphics.h"
#include "models.h"
#include "gui.h"

#include "graphics.inl"
#include "gui.inl"



namespace magnum { namespace sample {
using namespace magnum::types;
using magnum::graphics::Colour;
using magnum::graphics3d::Camera;
using magnum::graphics3d::Mode;
using magnum::graphics3d::PointLight;
using magnum::graphics3d::Model;
using magnum::graphics3d::StaticModel;
using magnum::graphics3d::ParameterBuffer;
using magnum::graphics3d::Context;
using magnum::graphics3d::Mesh;
using magnum::graphics3d::IndexMesh;
using magnum::graphics3d::MeshList;
using magnum::graphics3d::ColourShader;
using magnum::graphics3d::Shader;
using magnum::scene3d::Universe;
using magnum::scene3d::Root;
using magnum::scene3d::Locale;
using magnum::scene3d::CameraNode;
using magnum::scene3d::ModelNode;
using magnum::scene3d::PointLightNode;
using magnum::scene3d::Observer;

static Mode*			g_Mode;
static StaticModel*		g_Model;
static PointLight*		g_Light;
static ParameterBuffer* g_Parameters;
static Universe*		g_Universe;
static Locale*			g_Locale;
static Root*			g_Root;
static Observer*		g_Observer;
static CameraNode*		g_CameraNode;
static StringView*		g_ModelCountStringView;
static MeshList			g_MeshList;

static int				g_ModelCount;
static int				g_ModelSize;
static int				g_InsertLevel;

enum Command {
	Insert,
	Optimize
};


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void insertModel(int x,int y) {
	Transform3<float> t = Transform3<float>::Identity;
	t.translate(Vector3<float>(200.0f*x,300.0f*y,0));
	ModelNode* modelnode = new ModelNode(g_Model);
	modelnode->setTransform(t);
	g_Root->insert(modelnode);
	modelnode->release();

	g_ModelCount++;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void insertModelLevel() {
	if (g_InsertLevel == 0) {
		insertModel(0,0);
	} else {
		int i;
		for (i = -g_InsertLevel; i <= g_InsertLevel; i++)
			insertModel(i, -g_InsertLevel);
		for (i = -g_InsertLevel+1; i <= g_InsertLevel-1; i++)
			insertModel(-g_InsertLevel, i);
		for (i = -g_InsertLevel+1; i <= g_InsertLevel-1; i++)
			insertModel(g_InsertLevel, i);
		for (i = -g_InsertLevel; i <= g_InsertLevel; i++)
			insertModel(i, g_InsertLevel);
	}
	
	g_InsertLevel++;

	char buffer[40];
	sprintf(buffer, "Models: %d   Polys: %d", g_ModelCount, g_ModelCount*g_ModelSize);
	g_ModelCountStringView->setText(buffer);
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onInit() {
	g_InsertLevel = 0;
	g_ModelCount = 0;

	g_Mode = new Mode();
	g_Mode->setDepthTest(Mode::Less);
	g_Mode->setDepthWrite(true);

	// Create Universe
	g_Universe = new Universe();
	g_Locale = new Locale(g_Universe);
	g_Root = new Root();
	g_Locale->insert(g_Root);

	g_CameraNode = new CameraNode();
	g_CameraNode->setNearPlane(20);
	g_CameraNode->setFarPlane(1600);
	g_Root->insert(g_CameraNode);

	g_Observer = new Observer(g_Universe);
	g_Observer->attachCamera(g_CameraNode);

	PointLightNode* lightnode = new PointLightNode();
	lightnode->setPosition(Vector3<float>(900.0f,790.0f,400.0f));
	lightnode->setAmbient(Colour<float>(0.0f,0.0f,0.0f));
	lightnode->setDiffuse(Colour<float>(1.0f,1.0f,1.0f));
	lightnode->setSpecular(Colour<float>(0.3f,0.3f,0.3f));
//	g_Root->insert(lightnode);
//	lightnode->release();
	g_Light = (PointLight*)lightnode->getLight();
	g_Light->select();

	g_Model = (StaticModel*)create3ds();
	g_Model->getMeshes(&g_MeshList);
	g_ModelSize = 0;
	MeshList::Iterator miter;
	for (miter = g_MeshList.begin(); miter != g_MeshList.end(); ++miter) {
		g_ModelSize += ((IndexMesh*)*miter)->getRange().getSize() / 3;
	}

	// Create GUI
	Button* button;
	button = new Button(Rectangle<float>(10,10,170,40), Message(this, Insert), "Insert Model");
	button->setFont(m_Gui->getFont());
	getGui()->insert(button);

	button = new Button(Rectangle<float>(10,60,170,40), Message(this, Optimize), "Optimize");
	button->setFont(m_Gui->getFont());
	getGui()->insert(button);
	
	StringView* sview;
	sview = new StringView(Rectangle<float>(10,480,170,40), "Models: 0");
	sview->setAlignement(View::FollowBottom);
	sview->setFont(m_Gui->getFont());
	getGui()->insert(sview);
	g_ModelCountStringView = sview;


	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onExit() {
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
bool GraphicsView::onHandleMessage(const Message& msg) {
	switch (msg.getCommand()) {
		case Insert:
			insertModelLevel();
			return true;
		case Optimize:
			((Button*)msg.getPointer("source"))->disable();
			g_Model->optimize();
			return true;
	};
	return true;
}


/*----------------------------------------------------------------------------
----------------------------------------------------------------------------*/
void GraphicsView::onRender() {
	g_Mode->select();
	m_Frame->clear(FrameBuffer::ClearColour | FrameBuffer::ClearDepth | FrameBuffer::ClearStencil);
	m_Context->getLightStack()->enableLighting(LightStack::TwoSided | LightStack::LocalViewer);
	m_Context->setCulling(Context::CullBack);
	g_CameraNode->setTransform(getTransform().getInverse());

//	g_Observer->process();
	g_Universe->update();
	g_Observer->view();
	m_Gui->process();
	m_Gui->paint();
}

}; };

